# Copyright (c) 2017-2026 Juancarlo Añez (apalala@gmail.com)
# SPDX-License-Identifier: BSD-4-Clause
from __future__ import annotations

from pathlib import Path


PATH_TATSU_GRAMMAR = str(Path().cwd() / 'grammar/tatsu.ebnf')
