# Copyright (c) 2017-2026 Juancarlo Añez (apalala@gmail.com)
# SPDX-License-Identifier: BSD-4-Clause
from __future__ import annotations

import sys

from .cli import cling_main


if __name__ == "__main__":
    sys.exit(cling_main())
