# Copyright (c) 2017-2026 Juancarlo Añez (apalala@gmail.com)
# SPDX-License-Identifier: BSD-4-Clause
from __future__ import annotations

from .tool.cli import tatsu_main


if __name__ == '__main__':
    tatsu_main()
